@@ -380,7 +380,7 @@ rstto_image_viewer_class_init(RsttoImageViewerClass *viewer_class)
                                 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
                                 G_STRUCT_OFFSET (RsttoImageViewerClass, set_scroll_adjustments),
                                 NULL, NULL,
-                                rstto_marshal_VOID__OBJECT_OBJECT,
+                                _rstto_marshal_VOID__OBJECT_OBJECT,
                                 G_TYPE_NONE, 2,
                                 GTK_TYPE_ADJUSTMENT,
                                 GTK_TYPE_ADJUSTMENT);
@@ -881,20 +881,6 @@ correct_adjustments ( RsttoImageViewer *viewer )
     g_object_freeze_notify(G_OBJECT(viewer->hadjustment));
     g_object_freeze_notify(G_OBJECT(viewer->vadjustment));
 
-    /* Check if the image-size makes sense,
-     * if not, set the upper limits to 0.0
-     */
-    if ( (image_width < 1.0) || (image_height < 1.0) )
-    {
-        gtk_adjustment_set_upper (
-                viewer->hadjustment,
-                0.0 );
-        gtk_adjustment_set_upper (
-                viewer->vadjustment,
-                0.0 );
-        return;
-    }
-
     switch (viewer->priv->orientation)
     {
         case RSTTO_IMAGE_ORIENT_NONE:
@@ -1095,8 +1081,8 @@ paint_image (
                         M_PI*0.5);
                 cairo_translate (
                         ctx,
-                        0.0 - floor (gtk_adjustment_get_value (viewer->vadjustment)),
-                        floor (gtk_adjustment_get_value (viewer->hadjustment)));
+                        floor(0.0 - gtk_adjustment_get_value (viewer->vadjustment)),
+                        floor(gtk_adjustment_get_value (viewer->hadjustment)));
                 cairo_translate (
                         ctx,
                         0.0,
@@ -1488,9 +1474,28 @@ rstto_image_viewer_set_file (
             g_object_unref (viewer->priv->file);
             viewer->priv->file = NULL;
 
-            /* Reset the image-size to 0.0 */
-            viewer->priv->image_height = 0.0;
-            viewer->priv->image_width = 0.0;
+            g_object_freeze_notify(G_OBJECT(viewer->hadjustment));
+            g_object_freeze_notify(G_OBJECT(viewer->vadjustment));
+
+            gtk_adjustment_set_upper (
+                    viewer->hadjustment,
+                    0.0);
+            gtk_adjustment_set_value (
+                    viewer->hadjustment,
+                    0.0);
+
+            gtk_adjustment_set_upper (
+                    viewer->vadjustment,
+                    0.0);
+            gtk_adjustment_set_value (
+                    viewer->vadjustment,
+                    0.0);
+
+            /*
+             * Enable signals on the adjustments.
+             */
+            g_object_thaw_notify(G_OBJECT(viewer->vadjustment));
+            g_object_thaw_notify(G_OBJECT(viewer->hadjustment));
             
             gdk_window_invalidate_rect (
                     widget->window,
