@@ -52,12 +52,12 @@
 
 /* VOID:OBJECT,OBJECT (marshal.list:1) */
 void
-rstto_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
-                                   GValue       *return_value G_GNUC_UNUSED,
-                                   guint         n_param_values,
-                                   const GValue *param_values,
-                                   gpointer      invocation_hint G_GNUC_UNUSED,
-                                   gpointer      marshal_data)
+_rstto_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
+                                    GValue       *return_value G_GNUC_UNUSED,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint G_GNUC_UNUSED,
+                                    gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__OBJECT_OBJECT) (gpointer     data1,
                                                     gpointer     arg_1,
@@ -89,12 +89,12 @@ rstto_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
 
 /* VOID:UINT,BOXED (marshal.list:2) */
 void
-rstto_marshal_VOID__UINT_BOXED (GClosure     *closure,
-                                GValue       *return_value G_GNUC_UNUSED,
-                                guint         n_param_values,
-                                const GValue *param_values,
-                                gpointer      invocation_hint G_GNUC_UNUSED,
-                                gpointer      marshal_data)
+_rstto_marshal_VOID__UINT_BOXED (GClosure     *closure,
+                                 GValue       *return_value G_GNUC_UNUSED,
+                                 guint         n_param_values,
+                                 const GValue *param_values,
+                                 gpointer      invocation_hint G_GNUC_UNUSED,
+                                 gpointer      marshal_data)
 {
   typedef void (*GMarshalFunc_VOID__UINT_BOXED) (gpointer     data1,
                                                  guint        arg_1,
